/*
** http.h
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef   	HTTP_H_
# define   	HTTP_H_

#include <netinet/in.h>
#include <netdb.h>
/*
 * Public Interface
 */

#define HTTP_STD_PORT  80
#define HTTPS_STD_PORT 443
#define DEFAULT_HTTP_VERSION VERS_HTTP_11

#define HTTP_TRANSMIT_BUFFER_SIZE 2048

#define NB_METHOD   7
#define NB_HEADER   60
#define NB_VERSION  2

#define MAX_REQ_SIZE 6

typedef enum 
{
  REQ_UNKNOW = 0x00,
  REQ_GET    = 0x01,
  REQ_HEAD   = 0x02,
  REQ_POST   = 0x03,
  REQ_PUT    = 0x04,
  REQ_DELETE = 0x05,
  REQ_CONNECT = 0x06
}req_type;

typedef enum 
{
  FLD_UNKNOW              = 0x00,
  FLD_CONTENT_LENGHT      = 0x01,
  FLD_CONTENT_TYPE        = 0x02,
  FLD_DATE                = 0x03,
  FLD_FORWARDED           = 0x04,
  FLD_FROM                = 0x05,
  FLD_REFERER             = 0x06,
  FLD_IF_MODIFIED_SINCE   = 0x07,
  FLD_HOST                = 0x08,
  FLD_USER_AGENT          = 0x09,
  FLD_ACCEPT              = 0x0A,
  FLD_ACCEPT_LANGUAGE     = 0x0B,
  FLD_ACCEPT_ENCODING     = 0x0C,
  FLD_ACCEPT_CHARSET      = 0x0D,
  FLD_CONNECTION          = 0x0E,
  FLD_KEEP_ALIVE          = 0x0F,
  FLD_SERVER              = 0x10,
  FLD_LAST_MODIFIED       = 0x11,
  FLD_COOKIE              = 0x12,
  FLD_PROXY_CONNECTION    = 0x13,
  FLD_CONTENT_ENCODING    = 0x14,
  FLD_CONTENT_LANGUAGE    = 0x15,
  FLD_CONTENT_LOCATION    = 0x16,
  FLD_CONTENT_MD5         = 0x17,
  FLD_CONTENT_RANGE       = 0x18,
  FLD_ACCEPT_RANGES       = 0x19,
  FLD_ETAG                = 0x1A,
  FLD_CACHE_CONTROL       = 0x1B,
  FLD_SET_COOKIE          = 0x1C,
  FLD_TRANSFERT_ENCODING  = 0x1D,
  FLD_PRAGMA              = 0x1E,
  FLD_EXPIRES             = 0x1F,
  FLD_X_POWERED_BY        = 0x20,
  FLD_LOCATION            = 0x21,
  FLD_X_BENDER            = 0x22,
  FLD_X_PAD               = 0x23,
  FLD_VARY                = 0x24,
  FLD_X_ASPNET_VERSION    = 0x25,
  FLD_X_CACHE             = 0x26,
  FLD_EXPECT              = 0x27,
  FLD_IF_MATCH            = 0x28,
  FLD_IF_NONE_MATCH       = 0x29,
  FLD_IF_RANGE            = 0x2A,
  FLD_IF_UNMODIFIED_SINCE = 0x2B,
  FLD_P3P                 = 0x2C,
  FLD_TRAILER             = 0x2D,
  FLD_UPGRADE             = 0x2E,
  FLD_VIA                 = 0x2F,
  FLD_WARNING             = 0x30,
  FLD_TE                  = 0x31,
  FLD_X_FLASH_VERSION     = 0x32,
  FLD_AUTHOR              = 0x33,
  FLD_PERMANENTLY         = 0x34,
  FLD_PICS_LABEL          = 0x35,
  FLD_PROXY_AUTHENTICATE  = 0x36,
  FLD_PROXY_AUTHORIZATION = 0x37,
  FLD_MIME_VERSION        = 0x38,
  FLD_SOAP_ACTION         = 0x39,
  FLD_AUTHORIZATION       = 0x3A,
  FLD_WWW_AUTHENTICATE    = 0x3B
}field_type;

typedef enum 
  {
    VERS_HTTP_10            = 0x00,
    VERS_HTTP_11            = 0x01
  }http_version;


struct http_field_t
{
  field_type           type;
  char                *value;
  struct http_field_t *next;
};
typedef struct http_field_t http_field_t;

struct http_req_t
{
  req_type       req;
  char          *url;
  int            port;
  int            https;
  http_version   version;
  http_field_t  *lst_fields;
  int            body_lenght;
  unsigned char *body;
};
typedef struct http_req_t http_req_t;

struct http_resp_t
{ 
  http_version   version;
  int            status;
  http_field_t  *lst_fields;
  int            body_lenght;
  unsigned char *body;
};
typedef struct http_resp_t http_resp_t;


struct http_session_t
{
  int keep_alive;
  int first_connect;

  unsigned char use_cache_peer;
  unsigned char white_listed;

  char *fld_host_srv;

  char *host_cli, *host_srv;
  int unknow_host_cli, unknow_host_srv;
  int  sock_cli, sock_srv;
  struct sockaddr_in addr_srv, addr_cli;

  http_req_t  req_cli;
  http_req_t  req_srv;

  http_resp_t resp_srv;
  http_resp_t resp_cli;
};
typedef struct http_session_t http_session_t;


int http_parse_req_headers     (int sock,  http_req_t *req);
int http_parse_resp_headers    (int sock,  http_resp_t *resp);

int http_connect_srv(int sock_srv, http_req_t *req);

int http_transmit_req  (int *sock_srv, int sock_cli, http_req_t *req);
int http_transmit_resp (int sock_cli, int sock_srv, http_resp_t *resp);
void http_transmit_https(int sock_srv, int sock_cli);

#endif 	    /* !HTTP_H_ */
