/*
** vshlib.h
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef   	VSHLIB_H_
# define   	VSHLIB_H_


#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <signal.h>
#include <sys/stat.h>

/*
 * Public Interface  
 * 
 * Libc security Wrappers
 */

typedef unsigned char boolean;

#define NON_CRITICAL 0
#define CRITICAL     1

#define FAC_INFO     "VSH_INFO"
#define FAC_WARNING  "VSH_WARNING"
#define FAC_ERROR    "VSH_ERROR"

/* Network Primitives*/

int     vsh_socket(int domain, int type, int protocol);
int     vsh_setsockopt(int s, int level, int optname, const void *optval, socklen_t optlen);
int     vsh_bind(int sockfd, const struct sockaddr *my_addr, socklen_t addrlen);
int     vsh_listen(int sockfd, int backlog);
int     vsh_accept(int sockfd, struct sockaddr *addr, socklen_t *addrlen);
int vsh_connect(int sockfd, const struct sockaddr *serv_addr, socklen_t addrlen);
ssize_t vsh_send(int s, const void *buf, size_t len, int flags);
ssize_t vsh_recv(int s, void *buf, size_t len, int flags);
int     vsh_shutdown(int s, int how);

/* I/O Primitives */;

int     vsh_open(const char *pathname, int flags, boolean crit);
int     vsh_close(int fd);
ssize_t vsh_read(int fd, void *buf, size_t count);
ssize_t vsh_write(int fd, const void *buf, size_t count);
int     vsh_fstat(int filedes, struct stat *buf);


/* Process Primitives */

int     vsh_sigaction(int signum, 
		      const struct sigaction *act, 
		      struct sigaction *oldact);
void *  vsh_malloc(size_t size);
void    vsh_free (void *ptr);

void vsh_lock_init(char *pathname);
void vsh_lock_wait();
void vsh_lock_release();

/* String Primitives */

struct vsh_char_t
{
  char c;
  struct vsh_char_t *next;
};
typedef struct vsh_char_t vsh_char_t;

struct vsh_str_t
{
  int size;
  struct vsh_char_t *head;
  struct vsh_char_t *tail;
};
typedef struct vsh_str_t vsh_str_t;

void       vsh_str_init(vsh_str_t *str);
void       vsh_str_addchar (vsh_str_t *head, char c);
void       vsh_str_addnchar (vsh_str_t *head, char *buf, int lenght);
char      *vsh_str_inline  (vsh_str_t *head);
void       vsh_str_free(vsh_str_t *head);

/* Logs Primitives */

int        vsh_log(char *facility, char *message);

#endif 	    /* !VSHLIB_H_ */
