/*
** fstmatch.c
** 
** Copyright Julien Perez (C) 2005
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/


#include "fstmatch.h"
#include "vshlib.h"

/*
 *private interface
 */



static int fstmatch_find_value (fstmatch_token *token_head, char *pattern, void *store_value, int lenght)
{
  fstmatch_token *tmp_token;

  //printf("fstmatch_find\n");

  /* Arrivee a une feuille */
  if (!token_head)
    return FST_MATCH_KO;

  /* recherche dans le noeud fils */
  if (pattern[0] != '\0')
    for (tmp_token = token_head; tmp_token; tmp_token = tmp_token->next)
      {
	//printf("fstmatch_find test : token = [%c] pattern = [%c]\n", tmp_token->c, pattern[0]);
	if (tmp_token->c == pattern[0])
	  {
	    if (pattern[1] == '\0' && tmp_token->term)
	      {
		// *store_value = *(tmp_token->store_value);
		memcpy(store_value, tmp_token->store_value, lenght);
		return FST_MATCH_OK;
	      }
	    else
	      return fstmatch_find_value (tmp_token->childs, &pattern[1], store_value, lenght);
	  }
      }
 
  //printf("fstmatch_find no more token\n");
  return FST_MATCH_KO;
}


static int fstmatch_find(fstmatch_token *token_head, char *pattern)
{
  fstmatch_token *tmp_token;

  //printf("fstmatch_find\n");

  /* Arrivee a une feuille */
  if (!token_head)
    return FST_MATCH_OK;

  /* recherche dans le noeud fils */
  if (pattern[0] != '\0')
    for (tmp_token = token_head; tmp_token; tmp_token = tmp_token->next)
      {
	//printf("fstmatch_find test : token = [%c] pattern = [%c]\n", tmp_token->c, pattern[0]);
	if (tmp_token->c == pattern[0])
	  {
	    if (pattern[1] == '\0' && tmp_token->term)
	      return FST_MATCH_OK;
	    else
	      return fstmatch_find (tmp_token->childs, &pattern[1]);
	  }
      }
 
  //printf("fstmatch_find no more token\n");
  return FST_MATCH_KO;
}

static fstmatch_token *fstmatch_create_token(char c, unsigned char term, void *store_value)
{
  fstmatch_token *new;

  //printf("fstmatch_create_token\n");

  new = vsh_malloc(sizeof(fstmatch_token));

  new->c = c;
  new->term = term;
  new->next = NULL;
  new->childs = NULL;
  if (term)
    new->store_value = store_value;
  else
    new->store_value = NULL;
  return new;
}

static fstmatch_token *fstmatch_insert_token(fstmatch_token *current, char c, unsigned char term, void *store_value)
{
  fstmatch_token *tmp_token;
  
  //printf("fstmatch_insert_token\n");

  if (!current)
      exit(1);

  /*jai trouve le caractere, j'avance le current dessus */
  for (tmp_token = current->childs; tmp_token; tmp_token = tmp_token->next)
    if (tmp_token->c == c)
      {
	//printf(" - token found\n");
	if (term)
	  {
	    tmp_token->term = term;
	    tmp_token->store_value = store_value; // je met la valeur sur le DERNIER
	                                          // token 
	  }
	return tmp_token;
      }

  /*le noeud n'a pas encore de fils*/
  
  if (current->childs == NULL)
    {
      //printf(" - token no child\n");
      current->childs = fstmatch_create_token(c, term, store_value);
      return current->childs;
    }
  
  /*je suis en queue; je cree un fils en queue*/
  //printf(" - token add to childs\n");
  for (tmp_token = current->childs; tmp_token->next; tmp_token = tmp_token->next) ;
  tmp_token->next = fstmatch_create_token(c, term, store_value);
  return tmp_token->next;
}


/*
 * Public interface
 */

void  fstmatch_ctxt_init       (fstmatch_ctxt *ctxt)
{
  if (!ctxt)
    return ;

  ctxt->lst_tokens = NULL;
  ctxt->nb_token   = 0;
}

void  fstmatch_addpattern (fstmatch_ctxt *ctxt, char *pattern, void *store_value)
{
  int i, size;
  fstmatch_token *current;

  //printf("\nfstmatch_addpattern\n");

  if (!ctxt || !pattern)
    return ;
  if (strlen(pattern) == 0)
    return ;

  if (ctxt->lst_tokens == NULL)
    {
      if (pattern[1] == '\0')
	ctxt->lst_tokens = fstmatch_create_token(pattern[0], 1, store_value);
      else
        ctxt->lst_tokens = fstmatch_create_token(pattern[0], 0, store_value);
      current = ctxt->lst_tokens;
    }
  else
    {
      for (current = ctxt->lst_tokens; current; current = current->next)
	if (current->c == pattern[0])
	  break ;
      
      if (!current)
	{
	  for (current = ctxt->lst_tokens; current->next; current = current->next) ;
	  
	  if (pattern[1] == '\0')
	    current->next = fstmatch_create_token(pattern[0], 1, store_value);
	  else
	    current->next = fstmatch_create_token(pattern[0], 0, store_value);
	  current = current->next;
	}
    }

  size = strlen(pattern) - 1;

  for (i = 1; i < size; i++)
    current = fstmatch_insert_token(current, pattern[i], 0, store_value);
  fstmatch_insert_token(current, pattern[i], 1, store_value);
}


int   fstmatch_match      (fstmatch_ctxt *ctxt, char *pattern)
{
  int i; 

  if (!ctxt || !pattern)
    return FST_MATCH_KO;

  if (!ctxt->lst_tokens)
    return FST_MATCH_KO;

  if (strlen(pattern) == 0)
    return FST_MATCH_KO;

  for (i = 0; pattern[i]; i++)
    {
      //printf("fstmatch_match\n");
      if (fstmatch_find(ctxt->lst_tokens, &(pattern[i])) == FST_MATCH_OK)
	return FST_MATCH_OK;
    }

  return FST_MATCH_KO;
}


int fstmatch_search_value (fstmatch_ctxt *ctxt, char *pattern, void *store_value, int lenght)
{
  if (!ctxt || !pattern)
    return FST_MATCH_KO;

  if (!ctxt->lst_tokens)
    return FST_MATCH_KO;

  if (strlen(pattern) == 0)
    return FST_MATCH_KO;

  if (fstmatch_find_value(ctxt->lst_tokens, pattern, store_value, lenght) == FST_MATCH_OK)
	return FST_MATCH_OK;

  return FST_MATCH_KO;
}



void  fstmatch_ctxt_debug (fstmatch_ctxt *ctxt)
{
  fstmatch_token *current;

  for (current = ctxt->lst_tokens; current; current = current->next)
    printf("%c\n", current->c);
}
