/*
** http_connect_srv.c
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <netinet/in.h>
#include <netdb.h>
#include <sys/time.h>
#include <errno.h>
#include <syslog.h>

#include "http.h"
#include "http_peer.h"
#include "vshlib.h"
#include "conf.h"

extern http_session_t http_session;


static int do_connect_srv(char *host, int port)
{
  int i;
  struct timeval tval;
  int sock_srv; 
  struct hostent *he;
  struct sockaddr_in addr_srv;
  
  if (!host)
    return -2;

  /*Verify if the host is a IP or an hostname ... */
  if (isalpha(host[0]))
    {
      if ((he = gethostbyname(host))== NULL)
	return -1;
      addr_srv.sin_addr = *((struct in_addr *)he->h_addr);
    }
  else
    inet_aton(host, &addr_srv.sin_addr);

  sock_srv = vsh_socket(AF_INET, SOCK_STREAM, 0);

  addr_srv.sin_family = AF_INET;
  addr_srv.sin_port = htons(port);
  memset(&(addr_srv.sin_zero), '\0', 8);
  
  tval.tv_sec = 10;
  tval.tv_usec = 0;
  setsockopt(sock_srv, SOL_SOCKET, SO_RCVTIMEO, &tval, sizeof(struct timeval));
  
  if (vsh_connect(sock_srv, (struct sockaddr *)&addr_srv, sizeof(struct sockaddr)) < 0)
    {      
      vsh_close(sock_srv);
      return -2;
    }

  http_session.first_connect = 0;  
  memcpy(&(http_session.addr_srv), &addr_srv, sizeof(struct sockaddr_in));
  return sock_srv;
}


int http_connection_cache_peer (int sock)
{
  int i, nb_peer;
  int new_sock;
  http_peer *peer;

  if (http_session.first_connect || !http_session.keep_alive)
    {
      nb_peer = http_peer_getnb();
      for (i = 0; i < nb_peer; i++)
	{
	  peer = http_peer_get(i);
	  if ((new_sock = do_connect_srv(peer->hostname, peer->port)) > 0)
	    return new_sock;
	  else
	    syslog(LOG_NOTICE, "unable to connect to cache peer : %s:%d", peer->hostname, peer->port);
	}
      return new_sock;
    }
  return sock ;
}


int http_connect_srv(int sock_srv, http_req_t *req)
{
  int i;
  char  *host, *direct_connect;

  /* Process cache peers */
  if (http_session.use_cache_peer)
    {
      sock_srv = http_connection_cache_peer(sock_srv);
      if (sock_srv > 0)
	return sock_srv;
      
      /* "direct_connect" directive application*/
      if (direct_connect = conf_get_node(CONF_DIRECT_CONNECT))
	if (!direct_connect || direct_connect[0] != '1')
	  return sock_srv;

      syslog(LOG_NOTICE, "Direct Connection Enabled ...");
      http_session.use_cache_peer = 0;
      http_session.first_connect  = 1;
      http_session.keep_alive     = 0;
      http_session.fld_host_srv = NULL;
      sock_srv = 0;
    }

 
  /* Process Origin servers */
  host = strdup(http_method_search_field(req->lst_fields, FLD_HOST));
  if (!host)
    {
      sock_srv = -2;
      return ;
    }
  for (i = 0; host[i]; i++)
    if (host[i] == ':')
      {
	host[i] = '\0';
	break ;
      }

  /* Origin HTTP server connection */
  if (http_session.first_connect || 
      !http_session.keep_alive   ||
      !http_session.fld_host_srv ||
      strcmp(host, http_session.fld_host_srv) != 0)
    {
      if (http_session.fld_host_srv != NULL && strcmp(host, http_session.fld_host_srv) != 0)
	if (!http_session.first_connect)
	  {
	    if(vsh_shutdown(sock_srv, SHUT_RDWR) == 0)
	      vsh_close(sock_srv);
	    http_session.keep_alive = 1;
	    http_session.first_connect = 1;  
	  }
      sock_srv = do_connect_srv(host, req->port);

      if (http_session.fld_host_srv)
	vsh_free(http_session.fld_host_srv);
      http_session.fld_host_srv = host;
    }
  return sock_srv;
}
