/*
** http_methods.c
** Login : <julien@localhost>
** $Id$
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdio.h>
#include <stdlib.h>

#include "vshlib.h"
#include "http.h"
#include "fstmatch.h"


/*
 * Private Interface
 */

char *methods [NB_METHOD] = 
  {
    "UNKNOW",
    "GET",
    "HEAD",
    "POST",
    "PUT",
    "DELETE",
    "CONNECT"
  };

char *versions[NB_VERSION] =
  {
    "HTTP/1.0",
    "HTTP/1.1"
  };

char *headers [NB_HEADER] =
  {
    "unknow",
    "content-length",
    "content-type",
    "date",
    "forwarded",
    "from",
    "referer",
    "if-modified-since",
    "host",
    "user-agent",
    "accept",
    "accept-language",
    "accept-encoding",
    "accept-charset",
    "connection",
    "keep-alive",
    "server",
    "last-modified",
    "cookie",
    "proxy-connection",
    "content-encoding",
    "content-language",
    "content-location",
    "content-md5",
    "content-range",
    "accept-ranges",
    "etag",
    "cache-control",
    "set-cookie",
    "transfer-encoding",
    "pragma",
    "expires",
    "x-powered-by",
    "location",
    "x-bender",
    "x-pad",
    "vary",
    "x-aspnet-version",
    "x-cache",
    "expect",
    "if-match",
    "if-none-match",
    "if-range",
    "if-unmodified-since",
    "p3p",
    "trailer", 
    "upgrade",
    "via",
    "warning",
    "te",
    "x-flash-version",
    "author",
    "permanently",
    "pics-label",
    "proxy-authenticate",
    "proxy-authorization",
    "mime-version",
    "soapaction",
    "authorization",
    "www-authenticate"
  };


/*
 * Public Interface
 */

/* HTTP structures debug primitives */

void http_methods_req_debug(http_req_t *req)
{
  http_field_t *tmp_field;

  printf("Requete : [%s]\nUrl : [%s]\nPort : [%d]\nVersion : [%s]\n", 
	 methods[(int)req->req], 
	 req->url,
	 req->port,
	 versions[(int)req->version]);
  
  for (tmp_field = req->lst_fields; 
       tmp_field != NULL; 
       tmp_field = tmp_field->next) 
    printf("Field : [%s]\nValue : [%s]\n\n", 
	   headers[(int)tmp_field->type],
	   tmp_field->value);
}


void http_methods_resp_debug(http_resp_t *resp)
{
  http_field_t *tmp_field;

  printf("Response : [%d]\n", resp->status);
  
  for (tmp_field = resp->lst_fields; 
       tmp_field != NULL; 
       tmp_field = tmp_field->next)
    
    printf("Field : [%d] [%s]\nValue : [%s]\n\n",
	   (int)tmp_field->type,
	   headers[(int)tmp_field->type],
	   tmp_field->value);
}


/* HTTP Field checking primitives */

int   http_is_field (http_field_t *lst_fields, field_type type, char *value)
{
  http_field_t *tmp_field;

  for (tmp_field = lst_fields; tmp_field; tmp_field = tmp_field->next)
    if (tmp_field->type == type)
      if (!strcmp(tmp_field->value, value))
	return 1;
 
  return 0;
}


/* HTTP Field Manipulation */

http_field_t *http_methods_rm_field(http_field_t *lst_fields, http_field_t *field)
{
  http_field_t *tmp_field;

  if (lst_fields == field)
    {
      tmp_field  = lst_fields;
      lst_fields = lst_fields->next;
      vsh_free(tmp_field);
    }
  else
    {
      for (tmp_field = lst_fields; tmp_field;tmp_field = tmp_field->next)
	if (tmp_field->next == field)
	  {
	    tmp_field->next = field->next;
	    vsh_free(field);
	    break ;
	  }
    }
  return lst_fields;
}


http_field_t *http_methods_add_field(http_field_t *lst_fields, field_type type, char *value)
{
  int i;
  http_field_t *new_field, *tmp_field;

  if (!value || type > NB_HEADER || type < (field_type)0)
    return ;

  /* structure creation*/
  new_field = (http_field_t*)vsh_malloc(sizeof(http_field_t));
  new_field->type = type;
  new_field->value = (char *)strdup(value);
  new_field->next = NULL;

  /* add in list */
  if (lst_fields == NULL)
    return new_field;
  for (tmp_field = lst_fields; tmp_field->next != NULL; tmp_field = tmp_field->next) ;
  tmp_field->next = new_field;
  return lst_fields;
}

char *http_method_search_field(http_field_t *lst_field, field_type type)
{
  http_field_t *tmp_field;
  
  for (tmp_field = lst_field; tmp_field; tmp_field = tmp_field->next)
    if (tmp_field->type == type)
      return tmp_field->value;
  
  return NULL;
}


/* HTTP structures initialisation and destruction */

static void http_methods_free_fields(http_field_t  *head)
{
  http_field_t *tmp, *tmp2;

  tmp = head;
  while (tmp != NULL) 
    {
      tmp2 = tmp->next;
      vsh_free(tmp->value);
      vsh_free(tmp);
      tmp = tmp2;
    }
}

void http_methods_init_req(http_req_t *req)
{
  req->url         = NULL;
  req->lst_fields  = NULL;
  req->body_lenght = 0;
  req->body        = NULL;
}

void http_methods_init_resp(http_resp_t *resp)
{
  resp->lst_fields = NULL;
  resp->body_lenght = 0;
  resp->body = NULL;
}

void http_methods_free_req(http_req_t *req)
{
  vsh_free(req->url);
  vsh_free(req->body);
  http_methods_free_fields(req->lst_fields);
}


void http_methods_free_resp(http_resp_t *resp)
{
  vsh_free(resp->body);
  http_methods_free_fields(resp->lst_fields);
}



/* Fast identifying for header-type*/

static fstmatch_ctxt http_header_ctxt;

void http_methods_fstmatch_initheader ()
{
  int i, *store_value;

  fstmatch_ctxt_init(&http_header_ctxt);
  for (i = 0; i < NB_HEADER; i++)
    {
      store_value = vsh_malloc(sizeof(int));
      *store_value = i;
      fstmatch_addpattern(&http_header_ctxt, headers[i], store_value);
    }
}

int http_methods_fstmatch_getheadertype(char *header)
{
  int res, store_value;
  
  res = fstmatch_search_value    (&http_header_ctxt, header, &store_value, sizeof(int));
  if (res == FST_MATCH_OK)
    return store_value;
  return (int)FLD_UNKNOW;
}
