/*
** http_parse.c
** Login : <julien@localhost>
** $Id$
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdlib.h>
#include <ctype.h>

#include "vshlib.h"
#include "tokenlib.h"
#include "spy.h"

#include "http.h"
#include "http_methods.h"

/* 
 * Private Interface 
 */

extern char *methods  [NB_METHOD];
extern char *headers  [NB_HEADER];
extern char *versions [NB_VERSION];


static int http_parse_req_method(token_session_t *session, http_req_t *req)
{
  int j;
  char *buffer;

  buffer = token_get_simple(session);

  if (!buffer)
    return 0;

  // Search the method in the reference list 
  for (j = 0; j < NB_METHOD; j++)
    {
      if (!strcmp(buffer, methods[j]))
	{
	  vsh_log(FAC_INFO, buffer);
	  req->req = (req_type)j;
	  break ;
	}
    }
  if (j == NB_METHOD)
    return 0;

  vsh_free(buffer);
  return 1;
}

static int http_parse_req_url(token_session_t *session, http_req_t *req)
{
  /* default setting */
  req->port  = HTTP_STD_PORT;
  req->https = 0;
  
  req->url = token_get_simple(session);

  if (!req->url)
    return 0;

  vsh_log(FAC_INFO, req->url);
  return 1;
}

static int http_parse_req_version(token_session_t *session, http_req_t *req)
{
  int i;
  char *version;

  version = token_get_reqversion (session);
  if (!version)
    return 0;

  for (i = 0; i < NB_VERSION; i++)
    if (!strcmp(version, versions[i]))
      {
	req->version = (http_version)i;
	break ;
      }

  if (i == NB_VERSION)
    return 0;

  vsh_free(version);
  return 1;
}


static void http_parse_req_add_fields(http_req_t *req, char *name, char *value)
{
  field_type type;

  if (!value)
    return ;
  
  /*  Search of header type with the fstmatch technic */
  type = http_methods_fstmatch_getheadertype(name);

  req->lst_fields = http_methods_add_field(req->lst_fields, type, value);
}


static int http_parse_req_fields(token_session_t *session, http_req_t *req)
{
  int i, stop = 0;
  char *buffer_field_name;
  char *buffer_field_value;
  char *column;

  while (!stop)
    {
      /*parse header type*/
      buffer_field_name = token_get_headtype (session, &stop);
      if (stop)
	break ;
      else if (!buffer_field_name)
	return 0 ;

      /*parse column */
      if (!token_get_column (session))
	return 0;

      /* parse header value */
      buffer_field_value = token_get_headvalue (session);
      if (!buffer_field_value)
        {
	  vsh_free(buffer_field_name);
	  continue ;
	}

      /* insert header value in the list */
      for (i = 0; i < strlen(buffer_field_name); i++)
	buffer_field_name[i] = tolower(buffer_field_name[i]);
      for (i = 0; i < strlen(buffer_field_value); i++)
	buffer_field_value[i] = buffer_field_value[i];

      http_parse_req_add_fields(req, buffer_field_name, buffer_field_value);
    }

  return 1;
}


static int http_parse_req_body   (http_req_t *req)
{
  int i;
  char *lenght;

  lenght = http_method_search_field(req->lst_fields, FLD_CONTENT_LENGHT);
  if (lenght > 0)
    {
      // clean the lenght field
      for (i = strlen(lenght) - 1; i > 0 && isblank(lenght[i]); i--)
	lenght[i] = '\0';
      req->body_lenght = atoi(lenght);
    }
  else
    req->body_lenght = 0;
}


/* 
 * Public Interface 
 */


/*Main Primitive of HTTP request parsing*/

token_session_t *session_cli;
int http_parse_req_headers(int sock, http_req_t *req)
{
  session_cli = token_init(sock);

  if (!http_parse_req_method(session_cli, req))
    return 0;

  if (!http_parse_req_url(session_cli, req))
    return 0;

  if (!http_parse_req_version(session_cli, req))
    return 0;

  if (!http_parse_req_fields(session_cli, req))
    return 0;

  if (req->req == REQ_PUT || req->req == REQ_POST)
    http_parse_req_body(req);
  else
    {
      req->body = NULL;
      req->body_lenght = -1;
    }
  return 1;
}
