/*
** http_parse_resp.c
** Login : <julien@localhost>
** $Id$
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "vshlib.h"
#include "tokenlib.h"

#include "http.h"
#include "http_methods.h"

extern char *headers  [NB_HEADER];
extern char *versions [NB_VERSION];


/* 
 * Private Interface 
 */


static int http_parse_resp_version (token_session_t *session, http_resp_t *resp)
{
  int i;
  char *version;

  version = token_get_simple(session);

  if (!version)
    return 0;

  for (i = 0; i < NB_VERSION; i++)
    if (!strcmp(version, versions[i]))
      {
	resp->version = (http_version)i;
	break ;
      }

  if (i == NB_VERSION)
    return 0;

  vsh_free(version);
  return 1;
}


static int http_parse_resp_status (token_session_t *session, http_resp_t *resp)
{
  char *status;
  
  status = token_get_simple(session);

  if(!status)
    return 0;

  resp->status = atoi (status);
  vsh_free (status);  
  status = token_get_respstatus (session);
  vsh_free(status);
  return 1;
}


static void http_parse_resp_add_fields(http_resp_t *resp, char *name, char *value)
{
  field_type type;

  if (!value)
    return ;

  type = http_methods_fstmatch_getheadertype(name);
  // printf("name [%s] [%d]\n", name, (int)type);

  resp->lst_fields = http_methods_add_field(resp->lst_fields, type, value);
}


static int http_parse_resp_fields(token_session_t *session, http_resp_t *resp)
{
  int i, stop = 0;
  char *buffer_field_name;
  char *buffer_field_value;
  char *column;
  
  while (!stop)
    {
      /*parse header type*/
      buffer_field_name = token_get_headtype (session, &stop);
      if (stop)
	continue ;
      else if (!buffer_field_name)
	return 0;
      
      /*parse column */
      if (!token_get_column (session))
	return 0;

      /* parse header value */
      buffer_field_value = token_get_headvalue (session);
      if (!buffer_field_value)
	{
	  vsh_free(buffer_field_name);
	  continue ;
	}
	
      /* insert header value in the list */
      for (i = 0; i < strlen(buffer_field_name); i++)
	buffer_field_name[i] = tolower(buffer_field_name[i]);
      for (i = 0; i < strlen(buffer_field_value); i++)
	buffer_field_value[i] = buffer_field_value[i];

      http_parse_resp_add_fields(resp, buffer_field_name, buffer_field_value);
    }
  return 1;
}


/* 
 * Public Interface 
 */



/* Main Primitive of HTTP response headers parsing */

token_session_t *session_srv;
int http_parse_resp_headers(int sock, http_resp_t *resp)
{
  session_srv = token_init(sock);

  if (!http_parse_resp_version (session_srv, resp))
    return 0;
  if (!http_parse_resp_status  (session_srv, resp))
    return 0;
  if (!http_parse_resp_fields  (session_srv, resp))
    return 0;
  return 1;
}
