/*
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/


#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/time.h>
#include <errno.h>

#include "spy.h"
#include "vshlib.h"
#include "tokenlib.h"

#include "mime.h"
#include "http.h"
#include "http_methods.h"


extern char *methods  [NB_METHOD];
extern char *headers  [NB_HEADER];
extern char *versions [NB_VERSION];

extern http_session_t http_session;

extern token_session_t *session_srv;
extern token_session_t *session_cli;
extern spy_filter_content_t tabfilter_content [];


static void http_transmit_req_cmd(int sock_srv, http_req_t *req)
{
  int i;
  char *buf;
  vsh_str_t str_req;

  vsh_str_init(&str_req);
  for (i = 0; methods[(int)req->req][i]; i++)
    vsh_str_addchar(&str_req, methods[(int)req->req][i]);
  vsh_str_addchar(&str_req, ' ');

  for (i = 0; req->url[i]; i++)
    vsh_str_addchar(&str_req, req->url[i]);
  vsh_str_addchar(&str_req, ' ');

  for (i = 0; versions[(int)req->version][i]; i++)
    vsh_str_addchar(&str_req, versions[(int)req->version][i]);
  vsh_str_addchar(&str_req, '\r');
  vsh_str_addchar(&str_req, '\n');
  
  buf = vsh_str_inline(&str_req);
  vsh_send(sock_srv, buf, strlen(buf), 0);

  vsh_str_free(&str_req);
  vsh_free(buf);
}

static void http_transmit_req_headers(int sock_srv, http_req_t *req)
{
  http_field_t *tmp_field;

  for (tmp_field = req->lst_fields; tmp_field; tmp_field = tmp_field->next)
    {
      vsh_send(sock_srv, headers[(int)tmp_field->type], 
	       strlen(headers[(int)tmp_field->type]), 0);
      vsh_send(sock_srv, ": ", 2, 0);
      vsh_send(sock_srv, tmp_field->value, strlen(tmp_field->value), 0);
      vsh_send(sock_srv, "\r\n", 2, 0);
    }
  vsh_send(sock_srv, "\r\n", 2, 0);
}

static int http_transmit_req_block(int sock_srv, unsigned char *block, int block_size)
{
  int i;

  if (!http_session.white_listed)
    for (i = 0; tabfilter_content[i].name; i++)
      if (tabfilter_content[i].active)
	if (tabfilter_content[i].fct_filter(block, 
					    block_size, 
					    tabfilter_content[i].active_log) == 0)
	  return 0;	      
  vsh_send(sock_srv, block, block_size, 0);
  return 1;
}

static int http_transmit_req_content(int sock_cli, int sock_srv, http_req_t *req)
{
  int           i, size, lenght, total_size = 0;
  unsigned char *buffer_flush, buffer[HTTP_TRANSMIT_BUFFER_SIZE];
  char          *content_lenght, *content_type;


  content_type = http_method_search_field(req->lst_fields, FLD_CONTENT_TYPE);
  content_lenght = http_method_search_field(req->lst_fields, FLD_CONTENT_LENGHT);

  spy_begin_content_filter(content_type, POLICY_OUTFLOW);

  if (content_lenght)
    {
      for (i = strlen(content_lenght) - 1; i > 0 && isblank(content_lenght[i]); i--)
	content_lenght[i] = '\0';
    }

  if (content_lenght)
    lenght = atoi(content_lenght);
  else
    lenght = 0;

   /* cf : pb de apache mod_proxy */
  if (lenght < 0)
    return ;
  
  /* body content sending*/
  buffer_flush = token_get_flush(session_cli, &size);
  if (size)
    {
      if (!http_transmit_req_block(sock_srv, buffer_flush, size))
	return 0;
      total_size += size;
      if (lenght && total_size >= lenght)
	return 1;
    }
  /* on-the-fly filtering */
  do
    {
      size = HTTP_TRANSMIT_BUFFER_SIZE;
      size = vsh_recv(sock_cli, buffer, size, 0);

      if (size > 0)
	{
	  if (!http_transmit_req_block(sock_srv, buffer, size))
	    return 0;
	  total_size += size;
	  if (lenght && total_size >= lenght)
	    break ;
	}
    }while (size > 0);
  return 1;
}

int http_transmit_req(int *sock_srv, int sock_cli, http_req_t *req)
{
  char  *host, *content_type;
  unsigned char *buffer_flush, buffer[HTTP_TRANSMIT_BUFFER_SIZE];
  int size, total_size, i;
  
  // not https processing here
  if (req->https)
    return ;

  // send the request
  //   -> cmd
  http_transmit_req_cmd(*sock_srv, req);

  // -> headers
  http_transmit_req_headers(*sock_srv, req);

  // -> content
  if (req->req != REQ_PUT && req->req != REQ_POST)
    return ;
  return http_transmit_req_content(sock_cli, *sock_srv, req);
}
