/*
** http_transmit_resp.c
** Login : <julien@localhost>
** Started on  Sat Oct 15 08:30:52 2005 
** $Id$
** 
** Copyright (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/time.h>

#include "spy.h"
#include "vshlib.h"
#include "tokenlib.h"

#include "mime.h"
#include "http.h"
#include "http_methods.h"


extern char *methods  [NB_METHOD];
extern char *headers  [NB_HEADER];
extern char *versions [NB_VERSION];

extern http_session_t http_session;

extern token_session_t *session_srv;
extern token_session_t *session_cli;
extern spy_filter_content_t tabfilter_content [];


static void http_transmit_resp_status(int sock_cli, http_resp_t *resp)
{
  char           status[5];

  vsh_send(sock_cli, versions[(int)resp->version], 
	   strlen(versions[(int)resp->version]), 0);
  vsh_send(sock_cli, " ", 1, 0);

  if (resp->status < 0 || resp->status > 999)
    spy_die();
  sprintf(status, "%d ", resp->status);
  status[4] = '\0';
  vsh_send(sock_cli, status, strlen(status), 0);
  vsh_send(sock_cli, "\r\n", 2, 0);
}


static void http_transmit_resp_headers(int sock_cli, http_resp_t *resp)
{
  http_field_t  *tmp_field;
  
  for (tmp_field = resp->lst_fields; tmp_field; tmp_field = tmp_field->next)
    {
      vsh_send(sock_cli, headers[(int)tmp_field->type], 
	       strlen(headers[(int)tmp_field->type]), 0);
      vsh_send(sock_cli, ": ", 2, 0);
      vsh_send(sock_cli, tmp_field->value, strlen(tmp_field->value), 0);
      vsh_send(sock_cli, "\r\n", 2, 0);
    }
  vsh_send(sock_cli, "\r\n", 2, 0);
}


static int http_transmit_resp_block(int sock_cli, unsigned char *block, int block_size)
{
  int i;

  if (!http_session.white_listed)
    for (i = 0; tabfilter_content[i].name; i++)
      {
	if (tabfilter_content[i].active)
	  if (tabfilter_content[i].fct_filter(block, 
					      block_size, 
					      tabfilter_content[i].active_log) == 0)
	    
	    return 0;
      }
  vsh_send(sock_cli, block, block_size, 0);
  return 1;
}


static int http_transmit_resp_content(int sock_cli, int sock_srv, http_resp_t *resp)
{
  int           i, size, lenght, total_size = 0;
  unsigned char *buffer_flush, buffer[HTTP_TRANSMIT_BUFFER_SIZE];
  char          *content_lenght, *content_type;


  content_type = http_method_search_field(resp->lst_fields, FLD_CONTENT_TYPE);
  content_lenght = http_method_search_field(resp->lst_fields, FLD_CONTENT_LENGHT);

  if (content_lenght)
    {
      for (i = strlen(content_lenght) - 1; i > 0 && isblank(content_lenght[i]); i--)
	content_lenght[i] = '\0';
    }

  if (content_lenght)
    lenght = atoi(content_lenght);
  else
    lenght = 0;

  spy_begin_content_filter(content_type, POLICY_INFLOW);

   /* cf : pb de apache mod_proxy */
  if (lenght < 0)
    return ;
  
  if (sock_srv <= 0)
    {
      /* error sending */
      if (resp->body_lenght)
	vsh_send(sock_cli, resp->body, resp->body_lenght, 0);
    }
  else
    {
      /* body content sending*/
      buffer_flush = token_get_flush(session_srv, &size);
      if (size)
	{
	  if (!http_transmit_resp_block(sock_cli, buffer_flush, size))
	      return 0;
	  total_size += size;
	  if (lenght && total_size >= lenght)
	    return 1;
	}
      /* on-the-fly filtering */
      do
	{
	  size = HTTP_TRANSMIT_BUFFER_SIZE;
	  size = vsh_recv(sock_srv, buffer, size, 0);

	  if (size > 0)
	    {
	      if (!http_transmit_resp_block(sock_cli, buffer, size))
		return 0;
	      total_size += size;
	      if (lenght && total_size >= lenght)
		break ;
	    }
	}while (size > 0);
    }
  return 1;
}


int http_transmit_resp(int sock_cli, int sock_srv, http_resp_t *resp)
{
  // --> send status
  http_transmit_resp_status(sock_cli, resp);

  // --> send HTTP headers
  http_transmit_resp_headers(sock_cli, resp);

  // -> send body
  return http_transmit_resp_content(sock_cli, sock_srv, resp);
}
