/*
** main.c
** Login : <julien@localhost>
** $Id$
** 
** Copyright Julien Perez(C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <syslog.h>

#include "vshlib.h"
#include "conf.h"
#include "http.h"
#include "http_methods.h"
#include "spy.h"


#define HTTP_PROXY_PORT 8080
#define BACKLOG   20
#define DEFAULT_NB_PREFORK_CHILD 15

http_session_t http_session;
static int sock_srv;


void sigchld_handler(int s);
static struct sigaction sa;

static int tcplib_setup()
{
  int yes = 1;
  struct sockaddr_in srv_addr;
  char *conf_addr, *conf_port, *conf_backlog;

  /* Configuration Nodes using*/
  conf_addr    = conf_get_node(CONF_BIND);
  conf_port    = conf_get_node(CONF_PORT);
  conf_backlog = conf_get_node(CONF_BACKLOG);

  /*tcp setup*/
  sock_srv = vsh_socket(AF_INET, SOCK_STREAM, 0);
  vsh_setsockopt(sock_srv, SOL_SOCKET, SO_REUSEADDR,&yes,sizeof(int));
  
  srv_addr.sin_family = AF_INET;
  
  /*listen port configuraton*/
  if (!conf_port)
    srv_addr.sin_port = htons(HTTP_PROXY_PORT);
  else
    srv_addr.sin_port = htons(atoi(conf_port));

  /*IP adress listen configuration*/
  if (!conf_addr)
    srv_addr.sin_addr.s_addr = INADDR_ANY;
  else
    srv_addr.sin_addr.s_addr = inet_addr(conf_addr);

  memset(&(srv_addr.sin_zero), '\0', 8);

  vsh_bind(sock_srv, (struct sockaddr*)&srv_addr, sizeof(struct sockaddr));

  if (!conf_backlog)
    vsh_listen(sock_srv, BACKLOG);
  else
    vsh_listen(sock_srv, atoi(conf_backlog));

  return 1;
}


static void siglib_setup()
{ 
  /* No Zombie-process */
  sa.sa_handler = sigchld_handler;
  sigemptyset(&sa.sa_mask);
  sa.sa_flags = SA_RESTART;
  vsh_sigaction (SIGCHLD, &sa, NULL);
}

/* Prefork children logic */
static void make_antispyd_child()
{
  int sin_size;
  int new_fd;
  struct sockaddr_in addr_cli;

  if (!fork())
    while (1)
      {
	
	bzero(&addr_cli, sizeof(struct sockaddr));
	
	vsh_lock_wait();
	sin_size = sizeof(struct sockaddr_in);
	new_fd = vsh_accept(sock_srv, (struct sockaddr *)&addr_cli, &sin_size);
	vsh_lock_release();
	
	memcpy(&(http_session.addr_cli), &addr_cli, sizeof(struct sockaddr_in));
	spy_core(new_fd);
      }
}


/* Creating Pre-forked children */
static void srv_loop()
{
  int i;
  int nb_process;
  char *str_nb_process;

  str_nb_process = conf_get_node(CONF_PREFORK_PROCESS);
  if (!str_nb_process)
    nb_process = DEFAULT_NB_PREFORK_CHILD;
  else
    nb_process = atoi(str_nb_process);

  if (nb_process <= 0)
    nb_process = DEFAULT_NB_PREFORK_CHILD;

  // init fast header matching structure
  http_methods_fstmatch_initheader ();

  // init cache_peer processing
  if (http_peer_getnb() > 0)
    http_session.use_cache_peer = 1;
  else
    http_session.use_cache_peer = 0;

  vsh_lock_init("/tmp/lock.XXXXXX");
  for (i = 0; i < nb_process; i++)
    make_antispyd_child();

  // vsh_close(srv_fd);
  while (1)
    pause();
}


static int usage(char *prog)
{
  syslog(LOG_NOTICE, "usage: %s <config_file>", prog);
  return 1;
}


void sigchld_handler(int s)
{
  vsh_sigaction (SIGCHLD, &sa, NULL);

  make_antispyd_child();
  while(wait(NULL) > 0) ;
}

int main (int argc, char **argv)
{
  /* logs processing */
  setlogmask (LOG_UPTO (LOG_NOTICE));
#ifdef PACKAGE
  openlog (PACKAGE, LOG_CONS | LOG_PID | LOG_NDELAY, LOG_LOCAL1);
#else
  openlog ("antispyd", LOG_CONS | LOG_PID | LOG_NDELAY, LOG_LOCAL1);
#endif

  /* Configuration processing */
  if (argc != 2)
    return usage(argv[0]);
  
  if (!conf_parse_file(argv[1]))
    return 1;
  conf_log();

  /* Signal processing */
  siglib_setup();
  
  /* tcp configuration */
  tcplib_setup();
  
  /* Go Baby Go ... */
  srv_loop();
  return 0;
}
