/*
** http.c
** Login : <julien@localhost>
** $Id$
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/


#include <string.h>
#include <sys/socket.h>
#include <sys/time.h>

#include "http.h"
#include "vshlib.h"
#include "spy.h"

/*
 * Public Interface
 */

// HTTP headers filter Definition
spy_filter_t tabfilter_header [] =
  {
    {"BLOCK_SIGN",      1, 1, spy_sign_req, spy_sign_resp, spy_sign_policy_rule, spy_sign_init},
    {"BLOCK_URL",       1, 1, spy_block_req_url, NULL, spy_block_url_policy_rule, spy_block_url_init},
    {"BLOCK_TYPE",      1, 1, spy_block_req_type, spy_block_resp_type, spy_block_type_policy_rule, spy_block_type_init},
    {"BLOCK_SIZE",      1, 1, NULL, spy_block_resp_size, spy_block_size_policy_rule, spy_block_size_init},
    {"SHAPE_SHELLCODE", 1, 1, spy_shape_req_shellcode, spy_shape_resp_shellcode, spy_shape_shellcode_policy_rule, spy_shape_shellcode_init},
    {"SHAPE_COOKIE",    1, 1, spy_shape_req_cookie, spy_shape_resp_cookie, spy_shape_cookie_policy_rule, spy_shape_cookie_init},
    {"SHAPE_IDENTITY",  1, 1, spy_shape_req_identity, spy_shape_resp_identity, spy_shape_identity_policy_rule, spy_shape_identity_init},
    {NULL, 0, NULL, NULL, NULL, NULL}
  };

// HTTP content filter Definition
spy_filter_content_t tabfilter_content [] =
  {
    {"BLOCK_CONTENT",   0, 1, spy_block_content,        spy_block_content_begin_session, spy_block_content_policy_rule, spy_block_content_init},
    {"BLOCK_SIZE",      0, 1, spy_block_content_size,   spy_block_size_begin_session, spy_block_size_policy_rule,  spy_block_size_init},
    {"SHAPE_POPUP",     0, 1, spy_shape_popup,          spy_shape_popup_begin_session, spy_shape_popup_policy_rule, spy_shape_popup_init},
    {NULL, 0, NULL, NULL, NULL}
  };

extern http_session_t http_session;

void spy_die()
{
  http_methods_free_req (&(http_session.req_cli));
  http_methods_free_req (&(http_session.req_srv));
  http_methods_free_resp(&(http_session.resp_srv));
  http_methods_free_resp(&(http_session.resp_cli));

  if (http_session.fld_host_srv)
    {
      vsh_free(http_session.fld_host_srv);
      http_session.fld_host_srv = NULL;
    }  

  if (http_session.sock_srv > 0)
    {
      if (!vsh_shutdown(http_session.sock_srv, SHUT_RDWR))
	vsh_close(http_session.sock_srv);
    }
  
  if (http_session.sock_cli > 0)
    {
      if (!vsh_shutdown(http_session.sock_cli, SHUT_RDWR))
	vsh_close(http_session.sock_cli);
    }

  if (http_session.host_cli)
    vsh_free(http_session.host_cli);
  if (http_session.host_srv)
  vsh_free(http_session.host_srv);
}

void spy_begin_content_filter(char *content_type, int flow)
{
  int i;

  for (i = 0; tabfilter_content[i].name; i++)
    if (tabfilter_content[i].active)
      if (tabfilter_content[i].fct_filter_begin_session != NULL)
	tabfilter_content[i].fct_filter_begin_session (content_type, flow);

  return 0;
}

static void spy_init()
{
  http_session.host_srv = NULL;

  http_methods_init_req (&(http_session.req_cli));
  http_methods_init_req (&(http_session.req_srv));
  http_methods_init_resp(&(http_session.resp_srv));
  http_methods_init_resp(&(http_session.resp_cli));
}

static void spy_begin(int sock_cli)
{
  struct timeval tval;

  http_session.sock_cli = sock_cli;
  http_session.first_connect = 1;
  http_session.keep_alive = 0;
  http_session.host_cli = NULL;
  http_session.host_srv = NULL;
  http_session.fld_host_srv = NULL;
  http_session.unknow_host_cli = 0;
  http_session.unknow_host_srv = 0;
  http_session.white_listed = 0;

  tval.tv_sec = 10;
  tval.tv_usec = 0;
  setsockopt(http_session.sock_cli, SOL_SOCKET, SO_RCVTIMEO, &tval, sizeof(struct timeval));
}


/* 
 * Main Antipsyd Primitive
 *
 * -> Receive  HTTP request
 * -> Filter   HTTP request
 * -> Transmit HTTP request to origin server
 * -> Receive  HTTP response
 * -> Filter   HTTP response
 * -> Transmit HTTP response to client
 */

void spy_core (int sock_cli)
{
  int res = 1;

  spy_begin(sock_cli);

 spy_core_start:
  spy_init();

  /* -> HTTP Request Processing */
  if (!http_parse_req_headers(http_session.sock_cli, &(http_session.req_cli)))
    goto spy_core_end ;

  /* White List processing*/
  http_session.white_listed = spy_whitelst_check(http_session.req_cli.url);

  if ((res = spy_filter_req(&(http_session.req_cli), &(http_session.req_srv))) <= 0)
    {
      if (res == 0)
	spy_response_block(&(http_session.resp_cli));
      http_session.keep_alive = 0;
      goto spy_core_send_resp_cli;
    }
  
  http_session.sock_srv =  http_connect_srv(http_session.sock_srv, &(http_session.req_srv));
  if (http_session.sock_srv < 0)
    {
      if (http_session.sock_srv == -1)
	spy_response_unreach(&(http_session.resp_cli));
      if (http_session.sock_srv == -2)
	spy_response_unconnect(&(http_session.resp_cli));
      http_session.keep_alive = 0;
      goto spy_core_send_resp_cli;
    }
  
  
  if (!http_transmit_req(&(http_session.sock_srv), http_session.sock_cli, &(http_session.req_srv)))
  {
    spy_response_block(&(http_session.resp_cli));
    http_session.keep_alive = 0;
    res = 0;
    goto spy_core_send_resp_cli;
  }

  if (http_session.req_srv.https)
    {
      http_transmit_https(http_session.sock_srv, http_session.sock_cli);
      goto spy_core_end;
    }

  /* -> HTTP Response Processing */
  if (!http_parse_resp_headers(http_session.sock_srv, &(http_session.resp_srv)))
    goto spy_core_end ;
  
  if((res = spy_filter_resp(&(http_session.resp_srv), &(http_session.resp_cli))) <= 0)
    {
      if (res == 0)
	spy_response_block(&(http_session.resp_cli));
      http_session.keep_alive = 0;
      goto spy_core_send_resp_cli;
    }
  
  spy_core_send_resp_cli :
    
    if (res > 0)
      {
	if (!http_transmit_resp(http_session.sock_cli, http_session.sock_srv, &(http_session.resp_cli)))
	  http_session.keep_alive = 0;
      }
    else
      http_transmit_resp(http_session.sock_cli, 0, &(http_session.resp_cli));

  /* -> Keep-Alive Processing  */
  if (http_session.keep_alive)
    goto spy_core_start;

  spy_core_end :
    spy_die(); 
}
