/*
** spy_block_size.c
**
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "spy.h"

#include <stdlib.h>

#include "spy.h"
#include "conf.h"
#include "vshlib.h"
#include "http.h"
#include "mime.h"

struct spy_quota_size_t
{
  char  *mime_type;
  long   max_size;
  struct spy_quota_size_t *next;
};
typedef struct spy_quota_size_t spy_quota_size_t;

static spy_quota_size_t *lst_quota_inflow  = NULL;
static spy_quota_size_t *lst_quota_outflow = NULL;
static int intrack  = 0;
static int outtrack = 0;

static long spy_block_size_parsesize(char *buf_size)
{
  int i, size, multipl = -1;
  
  if (!buf_size)
    return -1;

  i = strlen(buf_size) - 1;
  buf_size[i] = tolower(buf_size[i]);

  if (buf_size[i] == 'o')
    multipl = 0;
  if (buf_size[i] == 'k')
    multipl = 1000;
  if (buf_size[i] == 'm')
    multipl = 1000000;
  if (buf_size[i] == 'g')
    multipl = 1000000000;

  if (multipl == -1)
    return -1;
  
  buf_size[i] = 0;
  size = atoi(buf_size);

  return size * multipl;
}

static void spy_block_size_track(int flow, int track)
{
  if (flow == POLICY_INFLOW)
    intrack = track;
  if (flow == POLICY_OUTFLOW)
    outtrack = track;
}

static spy_quota_size_t *spy_block_size_addquota(char *mime_type, long max_size, int flow)
{
  spy_quota_size_t **lst;
  spy_quota_size_t *new_quota, *tmp_quota;
  
  if (max_size < 0)
    return ;

  if (flow == POLICY_INFLOW)
    lst = &lst_quota_inflow;
  else if (flow == POLICY_OUTFLOW)
    lst = &lst_quota_outflow;
  else
    return ;

  new_quota = vsh_malloc(sizeof(spy_quota_size_t));
  new_quota->mime_type = strdup(mime_type);
  new_quota->max_size = max_size;
  new_quota->next = NULL;

  if (!(*lst))
    {
      *lst = new_quota;
      return ;
    }

  for (tmp_quota = *lst; tmp_quota->next; tmp_quota = tmp_quota->next) ; 
  tmp_quota->next = new_quota;
}



// DEBUG conf : file_type file_size
int  spy_block_size_policy_rule (char *rule)
{
  int        i, mime_index, flow;
  long       size;
  vsh_str_t  str;
  char      *first, *second, *third;

  // get first token
  for (i = 0; rule[i] && isblank(rule[i]); i++) ;
  if (!rule[i])
    return 0;

  vsh_str_init(&str);
  for (i; rule[i] && !isblank(rule[i]); i++) 
    vsh_str_addchar(&str, rule[i]);
  
  first = vsh_str_inline(&str);
  vsh_str_free(&str);

  if (!strcmp(first, "inflow"))
    flow = POLICY_INFLOW;
  else if (!strcmp(first, "outflow"))
    flow = POLICY_OUTFLOW;
  else
    flow = POLICY_ALLFLOW;

  // get second token
  for (i; rule[i] && isblank(rule[i]); i++) ;
  if (!rule[i])
    return 0;
  
  vsh_str_init(&str);
  for (i; rule[i] && !isblank(rule[i]); i++) 
    vsh_str_addchar(&str, rule[i]);

  second = vsh_str_inline(&str);
  vsh_str_free(&str);

  // trackin processing
  if (!strcmp(second, "track-on"))
    {
      spy_block_size_track(flow, 1);
      return 1;
    }
  if (!strcmp(second, "track-off"))
    {
      spy_block_size_track(flow, 0);
      return 1;
    }

  if (flow == POLICY_ALLFLOW)
    {
      mime_index  = get_mime_index (first);
      size        = spy_block_size_parsesize(second);
      if (mime_index != -1 && size != -1)
	{
	  spy_block_size_addquota (first, size, POLICY_INFLOW);
	  spy_block_size_addquota (first, size, POLICY_OUTFLOW);
	  return 1;
	}
      return 0;
    }

  // get third token
  for (i; rule[i] && isblank(rule[i]); i++) ;
  if (!rule[i])
    return 0;
  
  vsh_str_init(&str);
  for (i; rule[i] && !isblank(rule[i]); i++) 
    vsh_str_addchar(&str, rule[i]);

  third = vsh_str_inline(&str);
  vsh_str_free(&str);

  mime_index  = get_mime_index (second);
  size        = spy_block_size_parsesize(third);

  if (mime_index != -1 && size != -1)
    {
      spy_block_size_addquota (second, size, flow);
      return 1;
      
    }
  return 0;
}

int  spy_block_size_init()
{
  return 1;
}

int spy_block_resp_size (http_resp_t *resp, int log)
{
  spy_quota_size_t *tmp_quota;
  char *content_size, *content_type;
  int i, size;

  // trim content_size
  for (i = strlen(content_size) - 1; isblank(content_size[i]); i--)
    content_size[i] = 0;

  content_size = http_method_search_field(resp->lst_fields, FLD_CONTENT_LENGHT);
  content_type = http_method_search_field(resp->lst_fields, FLD_CONTENT_TYPE);

  // not enought information to process on this stage
  if (!content_type || !content_size)
    return 1;

  size       = atoi(content_size);

  for (tmp_quota = lst_quota_inflow; tmp_quota; tmp_quota = tmp_quota->next)
    if (strstr(content_type, tmp_quota->mime_type))
      if (tmp_quota->max_size < size)
	{
	  spy_log("BLOCK_SIZE", conf_get_node(CONF_LOG), 0);
	  return 0;
	}

  return 1;
}

static long tmp_size;
static long max_size;
int spy_block_size_begin_session(char *content_type, int flow)
{
  int mime_index;
  spy_quota_size_t **lst;
  spy_quota_size_t *tmp_quota;

  if ((flow == POLICY_INFLOW  && !intrack) ||
      (flow == POLICY_OUTFLOW && !outtrack))
    {
      max_size = -1;
      return ;
    }

  if (flow == POLICY_INFLOW)
    lst = &lst_quota_inflow;
  else 
    lst = &lst_quota_outflow;

  tmp_size = 0;

  for (tmp_quota = *lst; tmp_quota; tmp_quota = tmp_quota->next)
    if (strstr(content_type, tmp_quota->mime_type))
      {
	max_size = tmp_quota->max_size;
	return ;
      }
  max_size = -1;
}

int spy_block_content_size (void *data, int lenght, int log)
{
  if (max_size == -1)
    return 1;

  tmp_size += lenght;
  if (tmp_size > max_size)
    {
      spy_log("BLOCK_SIZE", conf_get_node(CONF_LOG), 0);
      return 0;
    }
  return 1;
}
