/*
** spy_block_type.c
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdlib.h>

#include "spy.h"
#include "conf.h"
#include "vshlib.h"
#include "http.h"
#include "http_methods.h"
#include "mime.h"

extern pair_t mime_types[];

static spy_key_t *type_keys = NULL;
static int type_default_policy = POLICY_ALLOW_ALL;


int  spy_block_type_init()
{
  return 0;
}

int  spy_block_type_policy_rule (char *rule)
{
  int i, res = 0;
  vsh_str_t str;
  char *order;

  for (i = 0; rule[i] && isblank(rule[i]); i++) ;
  if (!rule[i])
    return 0;

  vsh_str_init(&str);
  for (i; rule[i] && !isblank(rule[i]); i++) 
    vsh_str_addchar(&str, rule[i]);

  order = vsh_str_inline(&str);
  vsh_str_free(&str);
  if (!strcmp("allow-all", order))
    {
      type_default_policy = POLICY_ALLOW_ALL;
      res = 1;
    }
  if (!strcmp("block-all", order))
    {
      type_default_policy = POLICY_BLOCK_ALL;
      res = 1;
    }
  if (!strcmp("allow", order))
    {
      if (type_default_policy == POLICY_BLOCK_ALL)
	type_keys = spy_key_add(type_keys, &rule[i]);
      res = 1;
    }
  if (!strcmp("block", order))
    {
      if (type_default_policy == POLICY_ALLOW_ALL)
	type_keys = spy_key_add(type_keys, &rule[i]);
      res = 1;
    }

  vsh_free(order);
  return res;
}


int spy_block_resp_type(http_resp_t *resp, int log)
{
  int i, res;
  char *type;
  spy_key_t *tmp_key;

  res = 1;
  type = http_method_search_field(resp->lst_fields, FLD_CONTENT_TYPE);

  if(!type)
    return type_default_policy;

  for (tmp_key = type_keys; tmp_key; tmp_key = tmp_key->next)
    if (!strncmp(type, tmp_key->key, strlen(tmp_key->key)))
      {
	if (type_default_policy == POLICY_BLOCK_ALL)
	  return 1;
	else
	  {
	    if (log)
	      spy_log("BLOCK_TYPE", conf_get_node(CONF_LOG), 0);
	    return 0;
	  }
      }

  if (type_default_policy == POLICY_BLOCK_ALL)
    return 0;
  else
    return 1;
}


int spy_block_req_type(http_req_t *req, int log)
{
  int i, res;
  int mime_index;
  spy_key_t *tmp_key;

  /* Search extension in the URL*/
  for (i = 0; req->url[i] && req->url[i] != '?'; i++) ;
  if (!req->url[i])
    i--;

  for (i; i >= 0; i--)
    {
      if (req->url[i] == '.')
	break ;
      if (!isalnum (req->url[i]))
	return type_default_policy;
    }

  if (i == 0)
    return type_default_policy;

  /* Search the mime type corresponding with the extension */
  for (mime_index = 0; mime_types[mime_index].first; mime_index++)
    if (!strcmp(mime_types[mime_index].first, &(req->url[i])))
      break ;
  
  /* if mime type is unknow */
  if (!mime_types[mime_index].first)
    return type_default_policy;

  /* filtering */
  for (tmp_key = type_keys; tmp_key; tmp_key = tmp_key->next)
    if (!strncmp(mime_types[mime_index].second, 
		 tmp_key->key, strlen(tmp_key->key)))
      {
	if (type_default_policy == POLICY_BLOCK_ALL)
	  return 1;
	else
	  {
	    if (log)
	      spy_log("BLOCK_TYPE", conf_get_node(CONF_LOG), 1);
	    return 0;
	  }
      }
  if (type_default_policy == POLICY_BLOCK_ALL)
    return 0;
  else
    return 1;
  return res;
}
