/*
** spy_filter_url.c
** Login : <julien@localhost>
** $Id$
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdlib.h>
#include <fcntl.h>

#include "spy.h"
#include "conf.h"
#include "vshlib.h"
#include "http.h"
#include "fstmatch.h"


static int url_default_policy = POLICY_ALLOW_ALL;
static fstmatch_ctxt url_match_ctxt;


static void spy_block_url_insert_pattern(int i, char *rule)
{
  int j;

  // trim and insert the pattern
  for (i; rule[i] && isblank(rule[i]); i++) ;
  if (rule[i])
    {
      for (j = strlen(rule) - 1; j != i && isblank(rule[j]); j--) ;
      rule[j + 1] = 0;
      fstmatch_addpattern(&url_match_ctxt, &rule[i], NULL);
    }
}

static void spy_block_url_addfile(int i, char *filename)
{
  int j, fd, res, stop = 0;
  vsh_str_t str;
  char *buf_line;
  char c;

  for (i; filename[i] && isblank(filename[i]); i++) ;
  if (filename[i])
    {
      for (j = strlen(filename) - 1; j != i && isblank(filename[j]); j--) ;
      filename[j + 1] = 0;
    }
  else
    return ;

  /* insert the patterns from the file*/
  fd = vsh_open(&filename[i], O_RDONLY, 1);
  do
    {
      /* line extraction */
      vsh_str_init(&str);
      do {
	res = vsh_read(fd, &c, 1);
	if (c != '\n')
	  vsh_str_addchar(&str, c);
      }while (res > 0 && c != '\n');

      /* line parsing */
      if (str.size > 0)
	{
	  buf_line = vsh_str_inline(&str);
	  spy_block_url_insert_pattern(0, buf_line);
	  vsh_free(buf_line);
	}
      else
	if (res <= 0)
	  stop = 1 ;
      vsh_str_free(&str);
    }while (!stop && res > 0);
  vsh_close(fd);
}

int  spy_block_url_policy_rule (char *rule)
{
  static unsigned char url_is_ctxt_init = 0;

  int i, res = 0;
  vsh_str_t str;
  char *order;

  for (i = 0; rule[i] && isblank(rule[i]); i++) ;
  if (!rule[i])
    return 0;

  vsh_str_init(&str);
  for (i; rule[i] && !isblank(rule[i]); i++) 
    vsh_str_addchar(&str, rule[i]);

  order = vsh_str_inline(&str);
  vsh_str_free(&str);
  if (!strcmp("allow-all", order))
    {
      url_default_policy = POLICY_ALLOW_ALL;
      res = 1;
    }
  if (!strcmp("block-all", order))
    {
      url_default_policy = POLICY_BLOCK_ALL;
      res = 1;
    }
  if (!strcmp("allow", order))
    {
      if (url_default_policy == POLICY_BLOCK_ALL)
	{
	  if (!url_is_ctxt_init)
	    {
	      fstmatch_ctxt_init(&url_match_ctxt);
	      url_is_ctxt_init = 1;
	    }
	  spy_block_url_insert_pattern(i, rule);
	}
      res = 1;
    }
  if (!strcmp("block", order))
    {
      if (url_default_policy == POLICY_ALLOW_ALL)
	{
	  if (!url_is_ctxt_init)
	    {
	      fstmatch_ctxt_init(&url_match_ctxt);
	      url_is_ctxt_init = 1;
	    }
	  // trim and insert the pattern
	  spy_block_url_insert_pattern(i, rule);
	}
      res = 1;
    }
  if (!strcmp("file", order))
    spy_block_url_addfile(i, rule);

  vsh_free(order);
  return res;
}


int  spy_block_url_init()
{
  return 1;
}


int spy_block_req_url(http_req_t *req, int log)
{
  int i, res;
  char *string;

  res = fstmatch_match (&url_match_ctxt, req->url);
  if (res == FST_MATCH_OK)
    {
      if (url_default_policy == POLICY_BLOCK_ALL)
	return 1;
      else
	{
	  if (log)
	    spy_log("BLOCK_URL", conf_get_node(CONF_LOG), 1);
	  return 0;
	}
    }
  else
    {
      if (url_default_policy == POLICY_BLOCK_ALL)
	{
	  if (log)
	    spy_log("BLOCK_URL", conf_get_node(CONF_LOG), 1);
	  return 0;
	}
      else
	return 1;
    }
}
