/*
** spy_filter_req.c
** Login : <julien@localhost>
** $Id$
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/


#include <string.h>

#include "spy.h"

#include "vshlib.h"
#include "http.h"
#include "http_methods.h"

extern spy_filter_t tabfilter_header[];
extern http_session_t http_session;


static int spy_filter_url_extract_port(http_req_t *req_cli, http_req_t *req_srv, int url_cursor)
{
  vsh_str_t str_port;
  char     *buf_port;

  vsh_str_init(&str_port);
  for (url_cursor = url_cursor + 1; 
       req_cli->url[url_cursor] && isdigit(req_cli->url[url_cursor]); 
       url_cursor++)
    vsh_str_addchar (&str_port, req_cli->url[url_cursor]);
  buf_port = vsh_str_inline(&str_port);
  if (buf_port)
    {
      req_srv->port = atoi(buf_port);
      vsh_free(buf_port);
    }
  vsh_str_free(&str_port);
  return url_cursor;
}

static int spy_filter_url_extract_proto(http_req_t *req_cli, http_req_t *req_srv)
{
  int url_cursor = 0;

  if (!req_cli->url)
    return ;

  if (!strncmp("http://", req_cli->url, 7))
    {
      url_cursor += 7;
      req_srv->https = 0;
    }
  else if (!strncmp("https://", req_cli->url, 8))
    {
      url_cursor += 8;
      req_srv->https = 1;
      req_srv->port = HTTPS_STD_PORT;
    }
  return url_cursor;
}

static void spy_filter_host_extract_port(http_req_t *req_cli, http_req_t *req_srv)
{
  char     *host, *buf_port;
  int       host_cursor;
  vsh_str_t str_port;

  host = http_method_search_field(req_cli->lst_fields, FLD_HOST);
  if (!host)
    return ;
  for (host_cursor = strlen(host) - 1; host_cursor > 0 && isdigit(host[host_cursor]); host_cursor--) ;
  if (host[host_cursor] == ':')
    {
      vsh_str_init(&str_port);
      for (host_cursor = host_cursor + 1; host[host_cursor]; host_cursor++)
	vsh_str_addchar (&str_port, host[host_cursor]);
      buf_port = vsh_str_inline(&str_port);
      if (buf_port)
	{
	  req_srv->port = atoi(buf_port);
	  vsh_free(buf_port);
	}
      vsh_str_free(&str_port);
    }
}

static void spy_filter_resolv_url(http_req_t *req_cli, http_req_t *req_srv)
{
  int       url_cursor = 0;
  vsh_str_t str_port;
  char     *buf_port;

  req_srv->port = HTTP_STD_PORT;
  req_srv->https = 0;

  /* URL introspection */
  url_cursor = spy_filter_url_extract_proto(req_cli, req_srv);
  
  /* extraction of port no in the URL*/
  if (req_cli->req != REQ_CONNECT)
    for (url_cursor; req_cli->url && req_cli->url[url_cursor]; url_cursor++)
      {
	if (req_cli->url[url_cursor] == ':')
	  url_cursor = spy_filter_url_extract_port(req_cli, req_srv, url_cursor);	  
	if (req_cli->url[url_cursor] == '/')
	  {
	    req_srv->url = strdup(&req_cli->url[url_cursor]);
	    break ;
	  }
      }
  /* CONNECT Case processing, the url don't change */
  else
    for (url_cursor; req_cli->url && req_cli->url[url_cursor]; url_cursor++)
      {
	if (req_cli->url[url_cursor] == ':')
	  url_cursor = spy_filter_url_extract_port(req_cli, req_srv, url_cursor);
	req_srv->url = strdup(req_cli->url);
	req_srv->https = 1;
      }

  /* Host field introspection -  Port no Search */
  spy_filter_host_extract_port(req_cli, req_srv);
}


static void spy_filter_req_keep_alive(http_req_t *req_cli, http_req_t *req_srv)
{
  char *proxy_connection;
  field_type connection_type;

  if (http_session.use_cache_peer)
    connection_type = FLD_PROXY_CONNECTION;
  else
    connection_type = FLD_CONNECTION;

  proxy_connection = http_method_search_field(req_cli->lst_fields, FLD_PROXY_CONNECTION);
  if (!proxy_connection)
    {
      
      if (req_cli->version == VERS_HTTP_10)
	{
	  http_session.keep_alive = 0;
	  req_srv->lst_fields 
	    = http_methods_add_field(req_srv->lst_fields, connection_type, "close");
	}
      else
	{
	  http_session.keep_alive = 1;
	  req_srv->lst_fields = http_methods_add_field(req_srv->lst_fields, connection_type, "keep-alive");
	  req_srv->lst_fields = http_methods_add_field(req_srv->lst_fields, FLD_KEEP_ALIVE, "15");
	}    
    }
  else
    {
      if (!strcmp(proxy_connection, "close"))
	{
	  http_session.keep_alive = 0;
	  req_srv->lst_fields = http_methods_add_field(req_srv->lst_fields, connection_type, "close");
	}
      if (!strcmp(proxy_connection, "keep-alive"))
	{
	  http_session.keep_alive = 1;
	  req_srv->lst_fields = http_methods_add_field(req_srv->lst_fields, connection_type, "keep-alive");
	  req_srv->lst_fields = http_methods_add_field(req_srv->lst_fields, FLD_KEEP_ALIVE, "15");
	}
    }
}


static void spy_filter_translate_req(http_req_t *req_cli, http_req_t *req_srv)
{
  http_field_t *tmp_field;
  char *proxy_connection;

  /* Command Copy*/
  req_srv->req = req_cli->req;

  /* URL Deduction */
  spy_filter_resolv_url(req_cli, req_srv);
  if (http_session.use_cache_peer)
    {
      if (req_srv->url)
	vsh_free(req_srv->url);
      req_srv->url = strdup (req_cli->url);
    }

  /*copy de la version*/
  req_srv->version = req_cli->version;

  /* HTTP Header Insertion */
  for (tmp_field = req_cli->lst_fields; tmp_field; tmp_field = tmp_field->next)
    if (tmp_field->type != FLD_PROXY_CONNECTION && 
	tmp_field->type != FLD_KEEP_ALIVE && 
	tmp_field->type != FLD_PROXY_AUTHORIZATION &&
	tmp_field->type != FLD_CONNECTION)
      req_srv->lst_fields = http_methods_add_field(req_srv->lst_fields, tmp_field->type, tmp_field->value);
  
  /* keep_alive Processing */
  spy_filter_req_keep_alive(req_cli, req_srv);

  req_srv->body_lenght = req_cli->body_lenght;
  req_srv->body = NULL;
}

/* Main Primitive of HTTP Request Filtering */

int spy_filter_req(http_req_t *req_cli, http_req_t *req_srv)
{
  int i;

  /* HEADERS Request Filtering*/
  if (!http_session.white_listed)
    for (i = 0; tabfilter_header[i].name; i++)
      if (tabfilter_header[i].active && tabfilter_header[i].fct_filter_req != NULL)
	{
	  if (tabfilter_header[i].fct_filter_req(req_cli, tabfilter_header[i].active_log) == 0)
	    return 0;
	}

  /* Request Translation*/
  spy_filter_translate_req(req_cli, req_srv);

  return 1;
}

