/*
** spy_filter_resp.c
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <string.h>

#include "spy.h"

#include "vshlib.h"
#include "http.h"
#include "http_methods.h"

extern spy_filter_t tabfilter_header[];
extern http_session_t http_session;

static void spy_filter_resp_keep_alive(http_resp_t *resp_srv, http_resp_t *resp_cli)
{
  char *connection;

  if (resp_srv->status > 399 && resp_srv->status < 500)
    http_session.keep_alive = 0;
  
  if (resp_srv->status == 100)
    {
      http_session.keep_alive = 1;
      return ;
    }

  if (!http_session.keep_alive)
    resp_cli->lst_fields = http_methods_add_field(resp_cli->lst_fields, FLD_PROXY_CONNECTION, "close");
  else
    {
      connection 
	= http_method_search_field(resp_srv->lst_fields, FLD_CONNECTION);
      if (!connection)
	connection 
	= http_method_search_field(resp_srv->lst_fields, FLD_PROXY_CONNECTION);

      if (!connection)
	{
	  if (resp_srv->version == VERS_HTTP_10)
	    {
	      
	      http_session.keep_alive = 0;
	      resp_cli->lst_fields = http_methods_add_field(resp_cli->lst_fields, FLD_PROXY_CONNECTION, "close");
	    }
	  else
	    {
	      http_session.keep_alive = 1;
	      resp_cli->lst_fields = http_methods_add_field(resp_cli->lst_fields, FLD_PROXY_CONNECTION, "keep-alive");
	      resp_cli->lst_fields = http_methods_add_field(resp_cli->lst_fields, FLD_KEEP_ALIVE, "15");
	    } 
	}
      else
	{
	  if (!strcmp(connection,"close"))
	    {
	      http_session.keep_alive = 0;
	      resp_cli->lst_fields = http_methods_add_field(resp_cli->lst_fields, FLD_PROXY_CONNECTION, "close");
	    }
	  if (!strcmp(connection,"keep-alive"))
	    {
	      resp_cli->lst_fields = http_methods_add_field(resp_cli->lst_fields, FLD_PROXY_CONNECTION, "keep-alive");
	      resp_cli->lst_fields = http_methods_add_field(resp_cli->lst_fields, FLD_KEEP_ALIVE, "15");
	    }
	}
    }
}

static void spy_filter_translate_resp(http_resp_t *resp_srv, http_resp_t *resp_cli)
{
  http_field_t *tmp_field;
 
  /* Status Copy*/
  resp_cli->version = DEFAULT_HTTP_VERSION;
  resp_cli->status = resp_srv->status;
  
  /* HTTP Headers Insertion */
  for (tmp_field = resp_srv->lst_fields; tmp_field; tmp_field = tmp_field->next)
    if (tmp_field->type != FLD_CONNECTION && 
	tmp_field->type != FLD_KEEP_ALIVE &&
	tmp_field->type != FLD_PROXY_CONNECTION)
      resp_cli->lst_fields = http_methods_add_field(resp_cli->lst_fields, tmp_field->type, tmp_field->value);
  
  
  /* Keep alive processing */
  spy_filter_resp_keep_alive(resp_srv, resp_cli);
}



/* Main Primitive of HTTP Response Filtering */

int spy_filter_resp(http_resp_t *resp_srv, http_resp_t *resp_cli)
{
  int i, res;

  /* HTTP headers Response Filtering*/
  if (!http_session.white_listed)
    for (i = 0; tabfilter_header[i].name; i++)
      if (tabfilter_header[i].active && tabfilter_header[i].fct_filter_resp)
	{
	  vsh_log(FAC_INFO, tabfilter_header[i].name);
	  if ((res = tabfilter_header[i].fct_filter_resp(resp_srv, tabfilter_header[i].active_log)) < 1)
	    return res;
	}
  
  /* Response Translation*/
  spy_filter_translate_resp(resp_srv, resp_cli);

  return 1;
}

