/*
** spy_key.c
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdlib.h>
#include <string.h>

#include "spy.h"
#include "vshlib.h"

/*
 * Public Interface
 */

spy_key_t *spy_key_add(spy_key_t *keys, char *keyword)
{
  int i;
  vsh_str_t str;
  spy_key_t *new_key, *tmp_key;

  // lazy ...
  if (!keyword)
    return keys;

  // trim the keyword
  for (i = 0; keyword[i] && isblank(keyword[i]); i++) ;
  if (!keyword[i])
    return keys;

  vsh_str_init(&str);
  for (i; keyword[i] && !isblank(keyword[i]); i++)
    vsh_str_addchar(&str, keyword[i]);
  

  new_key = vsh_malloc(sizeof(spy_key_t));
  new_key->key = vsh_str_inline(&str);
  new_key->next = NULL;
  vsh_str_free(&str);

  if (!keys)
    return new_key;

  for (tmp_key = keys; tmp_key->next; tmp_key = tmp_key->next);
  tmp_key->next = new_key;
  return keys;
}


int spy_key_fnmatch(spy_key_t *keys, char *test)
{
  spy_key_t *tmp_key;
  
  
  for (tmp_key = keys; tmp_key; tmp_key = tmp_key->next)
    {
      if (strstr(test, tmp_key->key) != NULL)
	return 1;
    }
  return 0;
}
