/*
** spy_log.c
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <syslog.h>
#include <netinet/in.h>
#include <netdb.h>

#include "spy.h"
#include "http.h"
#include "vshlib.h"


extern http_session_t http_session;


static char *spy_log_add_hostname(struct sockaddr_in addr, vsh_str_t *str_log)
{
  struct hostent *host;

  host = gethostbyaddr(&(addr.sin_addr), sizeof(struct in_addr), AF_INET);
  if (host)
    {
      vsh_str_addnchar(str_log, host->h_name, strlen( host->h_name));
      return strdup(host->h_name);
    }
  return NULL;
}

static void spy_log_add_ip(struct sockaddr_in addr, vsh_str_t *str_log)
{
  char *ip_adress;

  ip_adress = inet_ntoa(addr.sin_addr);
  vsh_str_addnchar(str_log, ip_adress, strlen(ip_adress));
}


static int spy_log_resolv(char *filtername, char *log_format, vsh_str_t *str_log, int is_req)
{
  if (!log_format[0])
    return -1;
  
  if (log_format[0] == '%')
    {
      vsh_str_addchar(str_log, '%');
      return 0;
    }

  // url
  if (!strncmp(log_format, "ur", 2))
    {
      vsh_str_addnchar(str_log, http_session.req_cli.url, strlen(http_session.req_cli.url));
      return 1;
    }

  // ip client
  if (!strncmp(log_format, "ic", 2))
    {
      spy_log_add_ip(http_session.addr_cli, str_log);
      return 1;
    }

  // ip srv
  if (!strncmp(log_format, "is", 2))
    {
      // have to be already connected ...
      if (!http_session.first_connect)
	spy_log_add_ip(http_session.addr_srv, str_log);
      return 1;
    }
  
  // client host name
  if (!strncmp(log_format, "nc", 2))
    {
      if (http_session.unknow_host_cli)
	return 1;
      if (!http_session.host_cli)
	{
	  http_session.host_cli = spy_log_add_hostname(http_session.addr_cli, str_log);
	  if (!http_session.host_cli) // for any DNS query loop
	    http_session.unknow_host_cli = 1;
	}
      else
	vsh_str_addnchar(str_log, http_session.host_cli, strlen(http_session.host_cli));
      return 1;
    }

  // srv host name
  if (!strncmp(log_format, "ns", 2))
    {
      if (http_session.unknow_host_srv)
	return 1;
      if (http_session.fld_host_srv)
	{
	  vsh_str_addnchar(str_log, http_session.fld_host_srv, strlen(http_session.fld_host_srv));
	  return 1;
	}
      // have to be already connected ...
      if (!http_session.first_connect)
	{
	  if (!http_session.host_srv)
	    {
	      http_session.host_srv = spy_log_add_hostname(http_session.addr_srv, str_log);
	      if (!http_session.host_srv) // for any DNS query loop
		http_session.unknow_host_srv = 1;
	    }
	  else
	    vsh_str_addnchar(str_log, http_session.host_srv, strlen(http_session.host_srv));
	}
      return 1;
    }

  // filtername
  if (!strncmp(log_format, "fn", 2))
    {
      vsh_str_addnchar(str_log, filtername, strlen(filtername));
      return 1;
    }
  
  return -1;
}

void spy_log(char *filtername, char *log_format, int is_req)
{
  int i, res;
  vsh_str_t str_log;
  char *log_msg;
  
  if (!log_format)
    {
    return ;
    }

  vsh_str_init(&str_log);
  
  for (i = 0; log_format[i]; i++)
    if (log_format[i] == '%')
      {
	i++;
	res = spy_log_resolv(filtername, &log_format[i], &str_log, is_req);

	if (res < 0)
	  {
	    syslog (LOG_NOTICE, "bad log format [%s]", log_format);
	    spy_die();
	  }
	i += res ;
      }
    else
      vsh_str_addchar(&str_log, log_format[i]);
  

  log_msg = vsh_str_inline(&str_log);
  syslog (LOG_NOTICE, log_msg);
  
  vsh_free(log_msg);
  vsh_str_free(&str_log);
}
