/*
** spy_shape_popup.c
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <string.h>

#include "spy.h"
#include "conf.h"
#include "vshlib.h"
#include "http.h"
#include "http_methods.h"


static char *filter_popup[] =
{
  "text",
  "application/x-javascript",
  NULL
};


int spy_shape_popup_policy_rule (char *rule)
{
  return 0;
}

int spy_shape_popup_init()
{
  return 0;
}

static int popup_capable(char *content_type)
{
  int i;

  /*laisser le choix a l'utilisateur*/
  if (content_type == NULL)
    return 0;

  for (i = 0; filter_popup[i]; i++)
    if (strstr(content_type, filter_popup[i]))
      return 1;
  return 0;
}


static int popup_capable_content;
int spy_shape_popup_begin_session (char *content_type, int flow)
{
  if (flow == POLICY_OUTFLOW)
    {
      popup_capable_content = 0;
      return ;
    }
  popup_capable_content = popup_capable (content_type);
}


// TODO : non-ASCII charset not supported ...
int spy_shape_popup (void *data, int lenght, int log)
{
  unsigned char replace = 0;
  char         *start_p = NULL;
  char         *close_p = NULL;
  char         *p       = NULL;
  char         *body, *src;

  if (!popup_capable_content || lenght <= 0)
    return 1;

  body = (char*)vsh_malloc(lenght + 1);
  src = data;
  memcpy(body, src, lenght);
  body[lenght] = 0;

   /*
    * replace the window.open( with a harmless JavaScript replacement
    * (notice the two single quotes)
    */
   while ((start_p = strstr(body, "window.open(")) != NULL)
   {
      if (start_p)
      {
	strncpy(start_p, "1;''.concat(", 12);
	replace = 1;
	if (log)
	  spy_log("SHAPE_POPUP", conf_get_node(CONF_LOG), 0);
      }
   }

   while ((start_p = strstr(body, "alert(")) != NULL)
   {
      if (start_p)
      {
	strncpy(start_p, "1;''.concat(", 12);
	replace = 1;
	if (log)
	  spy_log("SHAPE_POPUP", conf_get_node(CONF_LOG), 0);
      }
   }

   while ((start_p = strstr(body, "confirm(")) != NULL)
   {
      if (start_p)
      {
	strncpy(start_p, "1;''.concat(", 12);
	replace = 1;
	if (log)
	  spy_log("SHAPE_POPUP", conf_get_node(CONF_LOG), 0);
       }
   }


   /*
    * replace the .resizeTo( with a harmless JavaScript replacement
    */
   while ((start_p = strstr(body, ".resizeTo(")) != NULL)
   {
      if (start_p)
      {
	strncpy(start_p, ".scrollTo(", 10);
	replace = 1;
	if (log)
	   spy_log("SHAPE_POPUP", conf_get_node(CONF_LOG), 0);
      }
   }

   /* 
    * Filter javascript functions in the body TAG
    */
   start_p = strstr(body, "<body");
   if (start_p)
   {
     close_p = strchr(start_p, '>');
     if (close_p)
       {
         /* we are now between <body and the ending > */
         p = strstr(start_p, "onUnload");
         if (p)
	   {
	     strncpy(p, "_nU_", 4);
	     replace = 1;
	     if (log)
	       spy_log("SHAPE_POPUP", conf_get_node(CONF_LOG), 0);
	   }
         p = strstr(start_p, "onLoad");
         if (p)
	   {
	     strncpy(p, "_nE_", 4);
	     replace = 1;
	     if (log)
	       spy_log("SHAPE_POPUP", conf_get_node(CONF_LOG), 0);
	   }
       }
   }
   
   if (replace)
     strncpy(data, (unsigned char*)body, lenght);
   vsh_free(body);
   return 1;
}
