/*
** spy_whitelst.c
** Login : <julien@localhost>
** Started on  Wed Oct 19 01:20:26 2005 
** $Id$
** 
** Copyright (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#include <stdlib.h>
#include <fcntl.h>
#include <syslog.h>

#include "spy.h"
#include "http.h"
#include "fstmatch.h"
#include "vshlib.h"

static fstmatch_ctxt whitelst_match_ctxt;


static char *spy_whitelst_extract_domain(char *url)
{
  int url_cursor = 0;;
  char *domain;
  vsh_str_t str;

  if (!url)
    return NULL;
  
  if (url[0] == '/')
    return NULL;

  if (strlen (url) >= 7 && !strncmp("http://", url, 7))
    url_cursor += 7;

  else if (strlen (url) >= 8 && !strncmp("https://", url, 8))
    url_cursor += 8;

  vsh_str_init(&str);
  for (url_cursor; url[url_cursor] && url[url_cursor] != '/'; url_cursor++)
    vsh_str_addchar(&str, url[url_cursor]);
  
  domain =  vsh_str_inline(&str);
  vsh_str_free(&str);
  return domain;
}

void spy_whitelst_add (char *domain)
{
  int i, j;
  static unsigned char white_is_ctxt_init = 0;

  if (!domain)
    return ;

  if (!white_is_ctxt_init)
    {
      fstmatch_ctxt_init(&whitelst_match_ctxt);
      white_is_ctxt_init = 1;
    }

  syslog(LOG_NOTICE, " white_list_domain [%s]", domain);

  for (i = 0; domain[i] && isblank(domain[i]); i++) ;
  if (domain[i])
    {
      for (j = strlen(domain) - 1; j != i && isblank(domain[j]); j--) ;
      domain[j + 1] = 0;
      fstmatch_addpattern(&whitelst_match_ctxt, &domain[i], NULL);
    }
}

void spy_whitelst_addfile(char *filename)
{
  int i, j, fd, res, stop = 0;
  vsh_str_t str;
  char *buf_line;
  char c;

  for (i = 0; filename[i] && isblank(filename[i]); i++) ;
  if (filename[i])
    {
      for (j = strlen(filename) - 1; j != i && isblank(filename[j]); j--) ;
      filename[j + 1] = 0;
    }
  else
    return ;

  /* insert the patterns from the file*/
  fd = vsh_open(&filename[i], O_RDONLY, 1);
  do
    {
      /* line extraction */
      vsh_str_init(&str);
      do {
	res = vsh_read(fd, &c, 1);
	if (c != '\n')
	  vsh_str_addchar(&str, c);
      }while (res > 0 && c != '\n');

      /* line parsing */
      if (str.size > 0)
	{
	  buf_line = vsh_str_inline(&str);
	  spy_whitelst_add(buf_line);
	  vsh_free(buf_line);
	}
      else
	if (res <= 0)
	  stop = 1 ;
      vsh_str_free(&str);
    }while (!stop && res > 0);
  vsh_close(fd);
}

int spy_whitelst_check(char *url)
{
  int res;
  char *domain;
  
  domain = spy_whitelst_extract_domain(url);  
  if (domain)
    {
      res = fstmatch_match (&whitelst_match_ctxt, domain);
      vsh_free(domain);
      if (res == FST_MATCH_OK)
	  return 1;
    }
  return 0;
}
