/*
** tokenlib.c
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "tokenlib.h"
#include "vshlib.h"

/*
 * Private Interface
 */

static int token_getbyte(token_session_t *session, void *c)
{
  if (session->cursor == LIBSESSION_BUFFER_SIZE || session->cursor >= session->size)
    {
      session->size = vsh_read(session->fd, session->buffer, LIBSESSION_BUFFER_SIZE);
      if (session->size <= 0)
	return 0;
      session->cursor = 0;
    }

  *((unsigned char*)c) = session->buffer[session->cursor];
  session->cursor++;

  return 1;
}


static int token_is_separator(char c)
{
  if (c == '\r')
    return 1;
  if (c == '\n')
    return 1;
  return 0;
}


/*
 * Public Interface
 */

token_session_t *token_init(int fd)
{
  token_session_t *session;

  session = (token_session_t *)vsh_malloc(sizeof(token_session_t));
  session->fd = fd;
  session->size = 0;
  session->column_passed = 0;
  session->cursor = LIBSESSION_BUFFER_SIZE;
  
  return session;
}


char *token_get_simple     (token_session_t *session)
{
  int res;
  char c, *token;
  vsh_str_t str_token;

  vsh_str_init(&str_token);
  while((res = token_getbyte(session, &c)) > 0)
    {
      if (!isblank(c))
	break;
    }
  if (res == 0)
    return NULL;

  do {
    if (!isblank(c))
      vsh_str_addchar (&str_token, c);
    else
      break ;
  }while((res = token_getbyte(session, &c)) > 0);

  if (res == 0)
    {
      vsh_str_free(&str_token);
      return NULL;
    }
  token = vsh_str_inline(&str_token);
  vsh_str_free(&str_token);
  return token ;
}

char *token_get_reqversion (token_session_t *session)
{
  int res;
  char c, *token;
  vsh_str_t str_token;

  vsh_str_init(&str_token);
  while((res = token_getbyte(session, &c)) > 0)
    {
      if (!isblank(c))
	break;
    }
  if (res == 0)
    return NULL;

  do {
    if (!token_is_separator(c))
      vsh_str_addchar (&str_token, c);
    else
      break ;
  }while((res = token_getbyte(session, &c)) > 0);
  
  if (res == 0)
    {
      vsh_str_free(&str_token);
      return NULL;
    }

  if (c == '\r')
    token_getbyte(session, &c);

  token = vsh_str_inline(&str_token);
  vsh_str_free(&str_token);
  return token ;
}

char *token_get_respstatus (token_session_t *session)
{
  return token_get_reqversion (session);
}


char *token_get_headtype   (token_session_t *session, int *stop)
{
  int res;
  char c, *token;
  vsh_str_t str_token;

  *stop = 0;
  vsh_str_init(&str_token);
  while((res = token_getbyte(session, &c)) > 0)
    {
      if (!isblank(c))
	break;
    }

  if (res == 0)
    return NULL;

  do {
    if (!isblank(c) && !token_is_separator(c) && c != ':')
      vsh_str_addchar (&str_token, c);
    else
      break ;
  }while((res = token_getbyte(session, &c)) > 0);

  if (c == ':')
    session->column_passed = 1;

  if (token_is_separator(c))
    {      
      *stop = 1;
      if (c == '\r')
	token_getbyte(session, &c);
      vsh_str_free(&str_token);
      return NULL;
    }

  token = vsh_str_inline(&str_token);
  vsh_str_free(&str_token);
  return token ;
}

char *token_get_column     (token_session_t *session)
{
  int res;
  char c;

  if (session->column_passed == 1)
    return "1";
   
  while((res = token_getbyte(session, &c)) > 0)
    if (!isblank(c))
      break;
  
  if (c != ':')
    return NULL;
  
  return "1";
}

char *token_get_headvalue  (token_session_t *session)
{
  return token_get_reqversion (session);
}



unsigned char *token_get_flush(token_session_t *session, int *lenght)
{
  *lenght = session->size - session->cursor;
  if (*lenght > 0)
    return &(session->buffer[session->cursor]);

  *lenght = 0;
  return NULL;
}

