/*
** vsh_lock.c
** Login : <julien@localhost>
** Started on  Thu Oct 13 16:32:08 2005 
** $Id$
** 
** Copyright (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <errno.h>

#include "vshlib.h"

static struct flock lock_it, unlock_it;
static int lock_fd = -1;

void vsh_lock_init(char *pathname)
{
  char *lock_file;

  if ((lock_file = strdup(pathname)) == NULL)
    {
      perror("strdup");
      exit(1);
    }
  
  if ((lock_fd = mkstemp(lock_file)) == -1)
    {
      perror("mkstemp");
      exit(1);
    }

  if (unlink(lock_file) == -1)
    {
      perror("unlink");
      exit(1);
    }

  lock_it.l_type = F_WRLCK;
  lock_it.l_whence = SEEK_SET;
  lock_it.l_start = 0;
  lock_it.l_len = 0;

  unlock_it.l_type = F_UNLCK;
  unlock_it.l_whence = SEEK_SET;
  unlock_it.l_start = 0;
  unlock_it.l_len = 0;

}

void vsh_lock_wait()
{
  int rc;

  while((rc = fcntl(lock_fd, F_SETLKW, &lock_it)) < 0)
    if (errno == EINTR)
      continue ;
    else
      {
	perror("fcntl");
	exit(1);
      }
}

void vsh_lock_release()
{
  if (fcntl(lock_fd, F_SETLKW, &unlock_it) < 0)
    {
      perror("fcntl");
      exit(1);
    }
}
